/*
 * BabyPhone.c
 *
 * main file for BabyPhone
 *
 * This wizard-generated code is based on code adapted from the
 * stationery files distributed as part of the Palm OS SDK 4.0.
 *
 * Copyright (c) 1999-2000 Palm, Inc. or its subsidiaries.
 * All rights reserved.
 */
 
#include <PalmOS.h>
#include <PalmOSGlue.h>
#include <Hs.h>

#include "BabyPhone.h"
#include "BabyPhone_Rsc.h"

/*********************************************************************
 * Entry Points
 *********************************************************************/

extern Err PhnLibDisconnect(UInt16 refNum, UInt16 line) PHN_LIB_TRAP (PhnLibTrapDisconnect); 


/*********************************************************************
 * Global variables
 *********************************************************************/

Boolean gblRunning = false;
SndStreamRef gblStreamID;
UInt32 gblEventTimeOut;

struct tagPrefsData
{
	Int32 nVolume;
	Int16 nTrigger;
	Boolean boolMakeACall;
	Boolean boolReceiveACall;
	Char strPhoneNumber[32];
};
struct tagPrefsData gblPrefs;

Char strVolumeLabel[16];
Char strTriggerLabel[16];

UInt16 PhoneLibRefNum;	// Phone library reference number

struct tagCallbackData
{
	UInt16 mittel[10];
	UInt16 maximum[10];
	Boolean bValid;
};
struct tagCallbackData gblCallbackData;

/*********************************************************************
 * Internal Constants
 *********************************************************************/

/* Define the minimum OS version we support */
#define ourMinVersion    sysMakeROMVersion(5,0,0,sysROMStageDevelopment,0)
#define kPalmOS20Version sysMakeROMVersion(2,0,0,sysROMStageDevelopment,0)

#define RETURN_FROM_CALL (sysAppLaunchCmdCustomBase+1)

#define FTR_MAKE_A_CALL      1
#define FTR_RETURN_FROM_CALL 2
#define FTR_SCAN_IS_STARTED  3
#define FTR_TRIGGER_COUNT    4
#define FTR_ALLOW_NUMBER     5
#define FTR_OLD_BRIGHTNESS   6
#define FTR_OLD_ALARMAMP     7
#define FTR_OLD_SYSAMP       8
#define FTR_OLD_MASTERAMP    9

#define ALARM_REF_SLEEP      1
#define ALARM_REF_WAKE_UP    2

/*********************************************************************
 * Internal Functions
 *********************************************************************/


/*
 * FUNCTION: GetObjectPtr
 *
 * DESCRIPTION:
 *
 * This routine returns a pointer to an object in the current form.
 *
 * PARAMETERS:
 *
 * formId
 *     id of the form to display
 *
 * RETURNED:
 *     address of object as a void pointer
 */

static void * GetObjectPtr(UInt16 objectID)
{
	FormType * frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}

/***********************************************************************
 *
 * FUNCTION:    SndStreamCallback
 *
 * DESCRIPTION: Sound callback for recording.
 *
 * PARAMETERS:	userData	- Pointer to user data.
 *				streamRef	- Reference to the sound stream 
 *				bufferP		- Data buffer
 *				bufferSizeP	- Pointer to the data buffer size
 *
 * RETURNED:	errNone is the function succeeded.
 *
 ***********************************************************************/

static Err SndStreamCallback(void *userDataP, SndStreamRef streamRef, void *bufferP, UInt32 bufferSize)
{
	struct tagCallbackData *myData = (struct tagCallbackData *)userDataP;
	Int8 *ptr = (Int8 *) bufferP;
	UInt32 mittel = 0;
	UInt32 maximum = 0;
	int i;

	// We expect the user data to be valid
	if( myData == NULL )
		return -1;
	
	for (i=0; i<bufferSize; i++)
	{
		int sample = ptr[i];
		if (sample < 0)
			sample = -sample;
		
		if (sample > maximum)
			maximum = sample;
		
		mittel += sample;
	}
	mittel /= bufferSize;

	myData->bValid = false;
	for (i=0; i<9; i++)
	{
		myData->mittel[i] = myData->mittel[i+1];
		myData->maximum[i] = myData->maximum[i+1];
	}
	myData->mittel[9] = mittel;
	myData->maximum[9] = maximum;
	myData->bValid = true;

	return 0;
}

static Boolean CheckForTreo()
{
	// check for Treo 600 to use HS extensions
	UInt32 deviceID, companyID;
	Err devErr, compErr;
	Boolean result = false;

	devErr = FtrGet(sysFtrCreator, sysFtrNumOEMDeviceID, &deviceID);
	compErr = FtrGet(sysFtrCreator, sysFtrNumOEMCompanyID, &companyID);
	if ((devErr == errNone) && (compErr == errNone))
	{
		if ((deviceID == 'H201') || (deviceID == 'H101'))
			result = true;
		else if ((deviceID == 'H202') || (deviceID == 'H102'))
			result = true;
		else if (deviceID == 'D052')
			result = true;
		else if (deviceID == 'D053')
			result = true;
		else if (deviceID == 'D060')
			result = true;
		else
		{
			UInt32 version = 0;
			Err err = FtrGet (hsFtrCreator, hsFtrIDVersion, &version);
			if (err == errNone)
				result = true;
		}
	}

	return result;
}

static void DisableTreoKeyguard()
{
	// disable the keyguard
	if (CheckForTreo())
	{
		UInt32 keyguard;
		HsAttrGet(hsAttrKeyboardLocked, 0, &keyguard);
		if (keyguard)
		{
			keyguard = 0;
			HsAttrSet(hsAttrKeyboardLocked, 0, &keyguard);
		}
	}
}

static void InitPrefsValues()
{
	gblPrefs.nVolume = 64;
	gblPrefs.nTrigger = 16;
	gblPrefs.boolMakeACall = true;
	gblPrefs.boolReceiveACall = true;
	StrCopy(gblPrefs.strPhoneNumber, "0");
}

static void LoadPrefsValues()
{
	UInt16 size = 0;
	if (PrefGetAppPreferences(appFileCreator, appPrefID, NULL, &size, 0) == appPrefVersionNum)
	{
		size = sizeof(gblPrefs);
		PrefGetAppPreferences(appFileCreator, appPrefID, &gblPrefs, &size, 0);
	}
}

static void SavePrefsValues()
{
	PrefSetAppPreferences(appFileCreator, appPrefID, appPrefVersionNum, &gblPrefs, sizeof(gblPrefs), 0);
}

static void myAlarmProc(UInt16 almProcCmd, SysAlarmTriggeredParamType *paramP)
{
	// disable the keyguard
	if (CheckForTreo())
	{
		UInt32 keyguard;
		HsAttrGet(hsAttrKeyboardLocked, 0, &keyguard);
		if (keyguard)
		{
			keyguard = 0;
			HsAttrSet(hsAttrKeyboardLocked, 0, &keyguard);
		}
	}

	if (paramP->ref == ALARM_REF_SLEEP)
	{
		UInt32 value = 0;
		if ((FtrGet(appFileCreator, FTR_SCAN_IS_STARTED, &value) == errNone) && value)
			SndStreamPause(value, true);
		EvtEnqueueKey(vchrHardPower, 0, commandKeyMask);
	}
	else if (paramP->ref == ALARM_REF_WAKE_UP)
	{
		UInt32 value = 0;
		if ((FtrGet(appFileCreator, FTR_SCAN_IS_STARTED, &value) == errNone) && value)
			SndStreamPause(value, false);
		EvtSetAutoOffTimer(ResetTimer, 0);
	}
}

static void ScheduleNextSleepAlarm(UInt32 time)
{
//	AlmSetProcAlarm(myAlarmProc, ALARM_REF_SLEEP, time);
}

static void ScheduleNextWakeUpAlarm(UInt32 time)
{
//	AlmSetProcAlarm(myAlarmProc, ALARM_REF_WAKE_UP, time);
}

static void DialANumber(Char *numberP)
{
	PhoneAppLaunchCmdDialPtr paramsP = NULL;
	UInt16 size = sizeof(PhoneAppLaunchCmdDialType);
	
	LocalID dbID;
	UInt16 cardNo;
	DmSearchStateType stateInfo;

	// Setup a parameter block so the Phone application automatically dials a phone number
	size += StrLen(numberP) + 1;

	paramsP = MemPtrNew (size);
	if (paramsP)
	{
		MemSet (paramsP, size, 0);

		paramsP->version = 1;
		paramsP->failLaunchCreator = appFileCreator;
		paramsP->failLaunchCode = RETURN_FROM_CALL;
		paramsP->dialMethod = PhoneAppDialMethodNormal;

		if (numberP)
		{
			paramsP->number = MemPtrNew(StrLen(numberP) + 1);
			StrCopy(paramsP->number, numberP);
			MemPtrSetOwner (paramsP->number, 0);
		}

		// set a safe alarm which calls our app, if our app is already up and running, we don't care
		SysCurAppDatabase(&cardNo, &dbID);
		AlmSetAlarm(cardNo, dbID, 0, TimGetSeconds() + 60, false);

		MemPtrSetOwner(paramsP, 0);
		if (DmGetNextDatabaseByTypeCreator(true, &stateInfo, sysFileTApplication, hsFileCPhone, true, &cardNo, &dbID) == errNone)
		{
			if (SysUIAppSwitch(cardNo, dbID, phoneAppLaunchCmdDial, paramsP) == errNone)
			{
				FtrSet(appFileCreator, FTR_MAKE_A_CALL, 1);
			}
		}
	}
}


static Boolean RedrawGraph(FormPtr frmP)
{
	int i, triggered = 0;
	RectangleType rect;
	RGBColorType rgbMaximum = {0, 0, 0, 255};
	RGBColorType rgbMittel = {0, 255, 0, 255};
	RGBColorType rgbTrigger = {0, 0, 0, 0};

	rect.topLeft.x = 25;
	rect.topLeft.y = 50;
	rect.extent.x = 110;
	rect.extent.y = 75;
	WinEraseRectangle(&rect, 0);

	if (gblCallbackData.bValid)
	{
		UInt32 value;
		
		WinSetForeColorRGB(&rgbMaximum, NULL);
		for (i=0; i<10; i++)
		{
			Int16 maximum = gblCallbackData.maximum[i];
			if (maximum > 64)
				maximum = 64;
			rect.topLeft.x = 30 + i * 10;
			rect.topLeft.y = 120 - maximum;
			rect.extent.x = 9;
			rect.extent.y = maximum;
			WinDrawRectangle(&rect, 0);
			
			if (maximum > gblPrefs.nTrigger)
				triggered++;
		}

		WinSetForeColorRGB(&rgbMittel, NULL);
		for (i=0; i<10; i++)
		{
			Int16 mittel = gblCallbackData.mittel[i];
			if (mittel > 64)
				mittel = 64;
			WinDrawLine(30 + i * 10, 120 - mittel, 30 + i * 10 + 9, 120 - mittel);
		}

		if (triggered > 4)
		{
			RGBColorType rgbRed = {0, 255, 0, 0};
			WinSetForeColorRGB(&rgbRed, NULL);
		}
		else if (triggered > 2)
		{
			RGBColorType rgbYellow = {0, 255, 255, 0};
			WinSetForeColorRGB(&rgbYellow, NULL);
		}
		else
		{
			RGBColorType rgbGreen = {0, 0, 255, 0};
			WinSetForeColorRGB(&rgbGreen, NULL);
		}

		rect.topLeft.x = 5;
		rect.topLeft.y = 100;
		rect.extent.x = 10;
		rect.extent.y = 10;
		WinDrawRectangle(&rect, 2);
		
		if (FtrGet(appFileCreator, FTR_TRIGGER_COUNT, &value) == errNone)
		{
			Char tmp[16];
			StrPrintF(tmp, "%03ld", value);
			WinDrawChars(tmp, StrLen(tmp), 5, 80);
		}
	}

	WinSetForeColorRGB(&rgbTrigger, NULL);
	WinDrawLine(30, 120 - gblPrefs.nTrigger, 130, 120 - gblPrefs.nTrigger);
	
	return (triggered > 4);
}

/*
 * FUNCTION: MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:
 *
 * frm
 *     pointer to the MainForm form.
 */

static void MainFormInit(FormType *frmP)
{
	FieldType *field;
	UInt16 fieldIndex;

	fieldIndex = FrmGetObjectIndex(frmP, MainNumberField);
	field = (FieldType *)FrmGetObjectPtr(frmP, fieldIndex);
	FrmSetFocus(frmP, fieldIndex);

	/* dont stack FldInsert calls, since each one generates a
	 * fldChangedEvent, and multiple uses can overflow the event queue */
	FldInsert(field, gblPrefs.strPhoneNumber, StrLen(gblPrefs.strPhoneNumber));

	SndStreamSetVolume(gblStreamID, gblPrefs.nVolume * 16);
	
	CtlSetValue(GetObjectPtr(MainVolumeSlider), gblPrefs.nVolume);
	CtlSetValue(GetObjectPtr(MainTriggerSlider), gblPrefs.nTrigger);

	StrPrintF(strVolumeLabel, "%d", (Int16) gblPrefs.nVolume);
	FldSetTextPtr((FieldPtr) GetObjectPtr(MainVolumeField), strVolumeLabel);
	FldDrawField((FieldPtr) GetObjectPtr(MainVolumeField));

	StrPrintF(strTriggerLabel, "%d", (Int16) gblPrefs.nTrigger);
	FldSetTextPtr((FieldPtr) GetObjectPtr(MainTriggerField), strTriggerLabel);
	FldDrawField((FieldPtr) GetObjectPtr(MainTriggerField));

	FrmSetControlValue(frmP, FrmGetObjectIndex(frmP, MainAllowCheck), gblPrefs.boolReceiveACall);
	FrmSetControlValue(frmP, FrmGetObjectIndex(frmP, MainCallCheck), gblPrefs.boolMakeACall);
	
	if (gblRunning)
	{
		SndStreamStart(gblStreamID);
		gblEventTimeOut = SysTicksPerSecond() / 10;
		CtlSetLabel(GetObjectPtr(MainStartStopButton), "Stop");
	}
}

/*
 * FUNCTION: MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:
 *
 * command
 *     menu item id
 */

static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;

	switch (command)
	{
	}

	return handled;
}

/*
 * FUNCTION: MainFormHandleEvent
 *
 * DESCRIPTION:
 *
 * This routine is the event handler for the "MainForm" of this 
 * application.
 *
 * PARAMETERS:
 *
 * eventP
 *     a pointer to an EventType structure
 *
 * RETURNED:
 *     true if the event was handled and should not be passed to
 *     FrmHandleEvent
 */

static Boolean MainFormHandleEvent(EventType * eventP)
{
	Boolean handled = false;
	FormType *frmP = FrmGetActiveForm();

	switch (eventP->eType) 
	{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			FrmDrawForm(frmP);
			MainFormInit(frmP);
			handled = true;
			break;
		
		case frmUpdateEvent:
			/* 
			 * To do any custom drawing here, first call
			 * FrmDrawForm(), then do your drawing, and
			 * then set handled to true. 
			 */
			break;
		
		case ctlRepeatEvent:
			if (eventP->data.ctlRepeat.controlID == MainVolumeSlider)
			{
				gblPrefs.nVolume = eventP->data.ctlRepeat.value;
				SndStreamSetVolume(gblStreamID, gblPrefs.nVolume * 16);
				StrPrintF(strVolumeLabel, "%d", (Int16) gblPrefs.nVolume);
				FldSetTextPtr((FieldPtr) GetObjectPtr(MainVolumeField), strVolumeLabel);
				FldDrawField((FieldPtr) GetObjectPtr(MainVolumeField));
			}
			else if (eventP->data.ctlRepeat.controlID == MainTriggerSlider)
			{
				gblPrefs.nTrigger = eventP->data.ctlRepeat.value;
				StrPrintF(strTriggerLabel, "%d", (Int16) gblPrefs.nTrigger);
				FldSetTextPtr((FieldPtr) GetObjectPtr(MainTriggerField), strTriggerLabel);
				FldDrawField((FieldPtr) GetObjectPtr(MainTriggerField));
			}
			RedrawGraph(frmP);
			break;
		
		case ctlSelectEvent:
			if (eventP->data.ctlSelect.controlID == MainStartStopButton)
			{
				if (FldGetTextPtr((FieldPtr) GetObjectPtr(MainNumberField)))
					StrCopy(gblPrefs.strPhoneNumber, FldGetTextPtr((FieldPtr) GetObjectPtr(MainNumberField)));
				gblRunning = !gblRunning;
				if (gblRunning)
				{
					UInt16 cardNo;
					LocalID dbID;
					Err err;

					SndStreamStart(gblStreamID);
					gblEventTimeOut = SysTicksPerSecond() / 10;
					CtlSetLabel(GetObjectPtr(MainStartStopButton), "Stop");
					FtrSet(appFileCreator, FTR_SCAN_IS_STARTED, gblStreamID);
					FtrSet(appFileCreator, FTR_TRIGGER_COUNT, 0);

					// Register for voice events.
					err = PhnLibRegister(PhoneLibRefNum, appFileCreator, phnServiceVoice);
					if (err != errNone)
					{
						Char tmp[64];
						StrPrintF(tmp, "err: %x", err);
						PhnLibClose (PhoneLibRefNum);
						FrmCustomAlert(PhnLibRegisterAlert, tmp, "", "");
					}

					// schedule the alarm every 2 minutes
					if (SysCurAppDatabase( &cardNo, &dbID ) == errNone)
					{
						UInt16 alarmAmp, sysAmp, masterAmp;
						UInt8 bright = SysLCDBrightness(false, 10);
						FtrSet(appFileCreator, FTR_OLD_BRIGHTNESS, bright);
						SysLCDBrightness(true, 1);
						SndGetDefaultVolume(&alarmAmp, &sysAmp, &masterAmp);
						FtrSet(appFileCreator, FTR_OLD_ALARMAMP, alarmAmp);
						FtrSet(appFileCreator, FTR_OLD_SYSAMP, sysAmp);
						FtrSet(appFileCreator, FTR_OLD_MASTERAMP, masterAmp);
						AlmSetAlarm(cardNo, dbID, 0, TimGetSeconds() + 2 * 60, false);
						SysNotifyRegister( cardNo, dbID, hsNotifyRemSleepRequestEvent, NULL, 0, NULL );
					}
				}
				else
				{
					UInt16 cardNo;
					LocalID dbID;
					Err err;
					int i;

					SndStreamStop(gblStreamID);
					gblEventTimeOut = evtWaitForever;
					gblCallbackData.bValid = false;
					for (i=0; i<10; i++)
					{
						gblCallbackData.mittel[i] = 0;
						gblCallbackData.maximum[i] = 0;
					}
					CtlSetLabel(GetObjectPtr(MainStartStopButton), "Start");
					FtrUnregister(appFileCreator, FTR_SCAN_IS_STARTED);
					if (SysCurAppDatabase( &cardNo, &dbID ) == errNone)
					{
						UInt16 alarmAmp = 0, sysAmp = 0, masterAmp = 0;
						UInt32 value = 100;
						FtrGet(appFileCreator, FTR_OLD_BRIGHTNESS, &value);
						SysLCDBrightness(true, value);
						FtrGet(appFileCreator, FTR_OLD_ALARMAMP, &value);
						alarmAmp = value;
						FtrGet(appFileCreator, FTR_OLD_SYSAMP, &value);
						sysAmp = value;
						FtrGet(appFileCreator, FTR_OLD_MASTERAMP, &value);
						masterAmp = value;
						SndSetDefaultVolume(&alarmAmp, &sysAmp, &masterAmp);
						AlmSetAlarm(cardNo, dbID, 0, 0, false);
						SysNotifyUnregister( cardNo, dbID, hsNotifyRemSleepRequestEvent, 0 );
					}

					// Deregister for voice events.
					err = PhnLibRegister(PhoneLibRefNum, appFileCreator, 0);
					if (err != errNone)
					{
						Char tmp[64];
						StrPrintF(tmp, "err: %x", err);
						PhnLibClose (PhoneLibRefNum);
						FrmCustomAlert(PhnLibRegisterAlert, tmp, "", "");
					}
				}
			}
			else if (eventP->data.ctlSelect.controlID == MainAllowCheck)
			{
				gblPrefs.boolReceiveACall = FrmGetControlValue(frmP, FrmGetObjectIndex(frmP, MainAllowCheck));
				if (gblPrefs.boolReceiveACall)
				{
					Char *strNumber = NULL;

					strNumber = FldGetTextPtr((FieldPtr) GetObjectPtr(MainNumberField));
					if (strNumber)
					{
						void *ptr = NULL;

						if (FtrPtrNew(appFileCreator, FTR_ALLOW_NUMBER, StrLen(strNumber) + 1, &ptr) == errNone)
							DmStrCopy(ptr, 0, strNumber);
					}
				}
				else
				{
					FtrPtrFree(appFileCreator, FTR_ALLOW_NUMBER);
				}
			}
			else if (eventP->data.ctlSelect.controlID == MainCallCheck)
			{
				gblPrefs.boolMakeACall = FrmGetControlValue(frmP, FrmGetObjectIndex(frmP, MainCallCheck));
			}
			break;
		
		case winEnterEvent:
			DisableTreoKeyguard();
			break;
		
		case nilEvent:
			if (RedrawGraph(frmP))
			{
				Char *strNumber = NULL;
				UInt32 value;

				if (FtrGet(appFileCreator, FTR_TRIGGER_COUNT, &value) == errNone)
				{
					value++;
					FtrSet(appFileCreator, FTR_TRIGGER_COUNT, value);
				}
				strNumber = FldGetTextPtr((FieldPtr) GetObjectPtr(MainNumberField));
				if ((strNumber != NULL) && gblPrefs.boolMakeACall)
					DialANumber(strNumber);
			}
			break;
	}
    
	return handled;
}

/*
 * FUNCTION: AppHandleEvent
 *
 * DESCRIPTION: 
 *
 * This routine loads form resources and set the event handler for
 * the form loaded.
 *
 * PARAMETERS:
 *
 * event
 *     a pointer to an EventType structure
 *
 * RETURNED:
 *     true if the event was handled and should not be passed
 *     to a higher level handler.
 */

static Boolean AppHandleEvent(EventType * eventP)
{
	UInt16 formId;
	FormType * frmP;

	if (eventP->eType == frmLoadEvent)
	{
		/* Load the form resource. */
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		/* 
		 * Set the event handler for the form.  The handler of the
		 * currently active form is called by FrmHandleEvent each
		 * time is receives an event. 
		 */
		switch (formId)
		{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

		}
		return true;
	}

	return false;
}

/*
 * FUNCTION: AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.
 */

static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do 
	{
		/* change timeout if you need periodic nilEvents */
		EvtGetEvent(&event, gblEventTimeOut);

		if (! SysHandleEvent(&event))
		{
			if (! MenuHandleEvent(0, &event, &error))
			{
				if (! AppHandleEvent(&event))
				{
					FrmDispatchEvent(&event);
				}
			}
		}
	} while (event.eType != appStopEvent);
}

/*
 * FUNCTION: AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * RETURNED:
 *     errNone - if nothing went wrong
 */

static Err AppStart(void)
{
	UInt32 value = 0;
	Err err;

	if (!CheckForTreo())
	{
		FrmAlert(DeviceIncompatibleAlert);
		return -1;
	}

	err = HsGetPhoneLibrary (&PhoneLibRefNum);
	if (err != errNone)
	{
		FrmAlert(PhoneLibraryAlert);
		return -1;
	}

	err = PhnLibOpen(PhoneLibRefNum);
	if (err != errNone)
	{
		FrmAlert(PhoneLibraryOpenAlert);
		return -1;
	}

	err = SndStreamCreate(&gblStreamID, sndInput, 8000, sndInt8, sndMono, SndStreamCallback, &gblCallbackData, 1024, false);
	if (err != errNone)
	{
		PhnLibClose (PhoneLibRefNum);
		FrmAlert(SndStreamCreateAlert);
		return -1;
	}
	
	InitPrefsValues();
	LoadPrefsValues();

	if ((FtrGet(appFileCreator, FTR_SCAN_IS_STARTED, &value) == errNone) && value)
		gblRunning = true;

	// check if we was recalled after a phone call
	if ((FtrGet(appFileCreator, FTR_RETURN_FROM_CALL, &value) == errNone) && value)
	{
		FtrUnregister(appFileCreator, FTR_RETURN_FROM_CALL);
		gblRunning = true;
	}
	{
		UInt16 cardNo;
		LocalID dbID;
		if (SysCurAppDatabase( &cardNo, &dbID ) == errNone)
		{
			SysNotifyRegister( cardNo, dbID, sysNotifyLateWakeupEvent, NULL, 0, NULL );
			SysNotifyRegister( cardNo, dbID, sysNotifySleepNotifyEvent, NULL, 0, NULL );
		}
	}

	return errNone;
}

/*
 * FUNCTION: AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 */

static void AppStop(void)
{
	if (gblRunning)
		SndStreamStop(gblStreamID);

	SndStreamDelete(gblStreamID);

	PhnLibClose (PhoneLibRefNum);

	SavePrefsValues();

	/* Close all the open forms. */
	FrmCloseAllForms();

	{
		UInt16 cardNo;
		LocalID dbID;
		if (SysCurAppDatabase( &cardNo, &dbID ) == errNone)
		{
			SysNotifyUnregister( cardNo, dbID, sysNotifyLateWakeupEvent, 0 );
			SysNotifyUnregister( cardNo, dbID, sysNotifySleepNotifyEvent, 0 );
		}
	}
}

/*
 * FUNCTION: RomVersionCompatible
 *
 * DESCRIPTION: 
 *
 * This routine checks that a ROM version is meet your minimum 
 * requirement.
 *
 * PARAMETERS:
 *
 * requiredVersion
 *     minimum rom version required
 *     (see sysFtrNumROMVersion in SystemMgr.h for format)
 *
 * launchFlags
 *     flags that indicate if the application UI is initialized
 *     These flags are one of the parameters to your app's PilotMain
 *
 * RETURNED:
 *     error code or zero if ROM version is compatible
 */

static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	/* See if we're on in minimum required version of the ROM or later. */
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
	{
		if ((launchFlags & 
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
		{
			FrmAlert (RomIncompatibleAlert);

			/* Palm OS versions before 2.0 will continuously relaunch this
			 * app unless we switch to another safe one. */
			if (romVersion < kPalmOS20Version)
			{
				AppLaunchWithCommand(
					sysFileCDefaultApp, 
					sysAppLaunchCmdNormalLaunch, NULL);
			}
		}

		return sysErrRomIncompatible;
	}

	return errNone;
}

/*
 * FUNCTION: PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 * 
 * PARAMETERS:
 *
 * cmd
 *     word value specifying the launch code. 
 *
 * cmdPB
 *     pointer to a structure that is associated with the launch code
 *
 * launchFlags
 *     word value providing extra information about the launch.
 *
 * RETURNED:
 *     Result of launch, errNone if all went OK
 */

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
	{
		case RETURN_FROM_CALL:
			FtrSet(appFileCreator, FTR_RETURN_FROM_CALL, 1);
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;

			/* 
			 * start application by opening the main form
			 * and then entering the main event loop 
			 */
			FrmGotoForm(MainForm);
			AppEventLoop();

			AppStop();
			break;

		// Phone event
		case phnLibLaunchCmdEvent:
			{
				PhnEventPtr phoneEvent = (PhnEventPtr)cmdPBP;
				if ((phoneEvent->eventType == phnEvtStartIncomingCall) ||
					(phoneEvent->eventType == phnEvtConnectInd))
				{
					UInt16 cardNo = 0;
					LocalID dbID = 0;
					UInt32 typeID = 0, crID = 0;
					UInt16 locPhoneLibRefNum;
					Char tmp[64];

					if (HsGetPhoneLibrary (&locPhoneLibRefNum) && (PhnLibOpen(locPhoneLibRefNum) == errNone))
					{
						SysCurAppDatabase(&cardNo, &dbID);
						DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &typeID, &crID);

//						if ((typeID == sysFileTApplication) && (crID == appFileCreator))
						{
							Boolean bAllowed = false;
							if (phoneEvent->data.info.service == phnServiceVoice)
							{
								// Example: on incoming phone call
								// someone is calling us, check the number and take the call or refuse it
								PhnAddressHandle caller = phoneEvent->data.info.caller;
								CharPtr number = NULL;
								UInt32 phnType;
								Char *ptr = NULL;

								HsAttrGet(hsAttrPhoneType, 0, &phnType);

								if (phnType == hsAttrPhoneTypeGSM)
									number = PhnLibGetField(locPhoneLibRefNum, caller, phnAddrFldPhone); // For GSM
								else
									number = PhnLibAPGetField(locPhoneLibRefNum, caller, phnAddrFldPhone); // For CDMA
								if ((FtrGet(appFileCreator, FTR_ALLOW_NUMBER, (UInt32 *) &ptr) == errNone) && ptr)
								{
									{
										Char tmp[64];
										StrPrintF(tmp, "%s", number);
										WinDrawChars(tmp, StrLen(tmp), 0, 15);
							
										SysTaskDelay(100);
									}

									if (StrCompare(ptr, number) == 0)
									{
										bAllowed = true;
									}
								}
							}
							if (!bAllowed)
							{
								char stackProtector[16];
								Err err = PhnLibDisconnect(PhoneLibRefNum, phoneEvent->data.info.lineNumber);
   								phoneEvent->acknowledge = true;
							}
						}
						PhnLibClose (locPhoneLibRefNum);
					}
				}
				else if (phoneEvent->eventType == phnEvtBusy)
				{
					UInt32 value = 0;
SndPlaySystemSound(sndAlarm);
//					if ((FtrGet(appFileCreator, FTR_MAKE_A_CALL, &value) == errNone) && value)
					{
						LocalID dbID;
						UInt16 cardNo;
						DmSearchStateType stateInfo;
						UInt16 locPhoneLibRefNum;

						if (HsGetPhoneLibrary (&locPhoneLibRefNum) && (PhnLibOpen(locPhoneLibRefNum) == errNone))
						{
							PhnAddressHandle caller = phoneEvent->data.info.caller;
							CharPtr number = NULL;
							UInt32 phnType;
							Char *ptr = NULL;

							HsAttrGet(hsAttrPhoneType, 0, &phnType);

							if (phnType == hsAttrPhoneTypeGSM)
								number = PhnLibGetField(locPhoneLibRefNum, caller, phnAddrFldPhone); // For GSM
							else
								number = PhnLibAPGetField(locPhoneLibRefNum, caller, phnAddrFldPhone); // For CDMA

							{
								Char tmp[64];
								StrPrintF(tmp, "%s", number);
								WinDrawChars(tmp, StrLen(tmp), 0, 15);
							
								SysTaskDelay(100);
							}

							{
								char stackProtector[16];
								Err err = PhnLibDisconnect(PhoneLibRefNum, phoneEvent->data.info.lineNumber);
								phoneEvent->acknowledge = true;
							}

							PhnLibClose (locPhoneLibRefNum);
						}

						// the call was busy, return to our app
						FtrUnregister(appFileCreator, FTR_MAKE_A_CALL);

						DmGetNextDatabaseByTypeCreator(true, &stateInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
						SysUIAppSwitch(cardNo, dbID, RETURN_FROM_CALL, NULL);
					}
				}
				else if (phoneEvent->eventType == phnEvtDisconnectInd)
				{
					UInt32 value = 0;
//					if ((FtrGet(appFileCreator, FTR_MAKE_A_CALL, &value) == errNone) && value)
					{
						LocalID dbID;
						UInt16 cardNo;
						DmSearchStateType stateInfo;

						// our call was finished, we should return to our app
						FtrUnregister(appFileCreator, FTR_MAKE_A_CALL);
						
						// FIXME: add a 10 sec procedure alarm and then enqueue the key
						EvtEnqueueKey(vchrRockerCenter, 0, commandKeyMask);

						DmGetNextDatabaseByTypeCreator(true, &stateInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
						SysUIAppSwitch(cardNo, dbID, RETURN_FROM_CALL, NULL);
					}
				}
				else if (phoneEvent->eventType == phnEvtProgress)
				{
					UInt32 value = 0;
					if ((FtrGet(appFileCreator, FTR_MAKE_A_CALL, &value) == errNone) && value)
					{
						UInt16 locPhoneLibRefNum;

						if (HsGetPhoneLibrary (&locPhoneLibRefNum) && (PhnLibOpen(locPhoneLibRefNum) == errNone))
						{
							// reregister for voice events.
							PhnLibRegister(locPhoneLibRefNum, appFileCreator, phnServiceVoice);
							PhnLibClose (locPhoneLibRefNum);
						}
						// acknowledge, we are responsible
						phoneEvent->acknowledge = true;
					}
				}
				else if (phoneEvent->eventType == phnEvtConnectConf)
				{
					UInt32 value = 0;
					if ((FtrGet(appFileCreator, FTR_MAKE_A_CALL, &value) == errNone) && value)
					{
						UInt16 cardNo;
						LocalID dbID;
						if (SysCurAppDatabase( &cardNo, &dbID ) == errNone)
						{
							// someone has answered our call, remove the alarm
							AlmSetAlarm(cardNo, dbID, 0, 0, false);
						}
					}
				}
				else if (phoneEvent->eventType == phnEvtUpdate)
				{
					UInt32 value = 0;
					if ((FtrGet(appFileCreator, FTR_MAKE_A_CALL, &value) == errNone) && value)
					{
						FrmAlert(RomIncompatibleAlert);
					}
				}
			}
			break;
		
		case sysAppLaunchCmdNotify:
			{
				UInt32 value = 0;
				SysNotifyParamType *snpt = (SysNotifyParamType *) cmdPBP;

				if (snpt->notifyType == sysNotifyLateWakeupEvent)
				{
					if ((FtrGet(appFileCreator, FTR_SCAN_IS_STARTED, &value) == errNone) && value)
						ScheduleNextSleepAlarm(TimGetSeconds() + 15);
				}
				else if (snpt->notifyType == sysNotifySleepNotifyEvent)
				{
					if ((FtrGet(appFileCreator, FTR_SCAN_IS_STARTED, &value) == errNone) && value)
						ScheduleNextWakeUpAlarm(TimGetSeconds() + 60);
				}
				else if (snpt->notifyType == hsNotifyRemSleepRequestEvent)
				{
					SleepEventParamType *sept = snpt->notifyDetailsP;
					sept->deferSleep++;
				}
			}
			break;
		
		case sysAppLaunchCmdAlarmTriggered:
			{
				LocalID dbID;
				UInt16 cardNo;
				DmSearchStateType stateInfo;
				UInt32 value = 0;

				DmGetNextDatabaseByTypeCreator(true, &stateInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
				if ((FtrGet(appFileCreator, FTR_SCAN_IS_STARTED, &value) == errNone) && value)
				{
					SysUIAppSwitch(cardNo, dbID, RETURN_FROM_CALL, NULL);
				}
				// reschedule the alarm every 2 minutes
				AlmSetAlarm(cardNo, dbID, 0, TimGetSeconds() + 2 * 60, false);
			}
			break;
	}

	return errNone;
}
